<?php

declare(strict_types=1);

namespace DigiComp\PhpCodesniffer\DigiComp\Sniffs\ClassNames;

use PHP_CodeSniffer\Files\File;
use PHP_CodeSniffer\Sniffs\Sniff;
use SlevomatCodingStandard\Helpers\TokenHelper;

/**
 * Class AbstractMissingPrefixSniff is a blueprint for checking a missing prefix in a class name.
 */
abstract class AbstractMissingPrefixSniff implements Sniff
{
    /**
     * @inheritDoc
     */
    public function process(File $phpcsFile, $stackPtr)
    {
        $tokens = $phpcsFile->getTokens();

        $classNamePointer = TokenHelper::findNext(
            $phpcsFile,
            \T_STRING,
            $stackPtr + 1,
            $tokens[$stackPtr]['scope_opener']
        );
        $classNameToken = $tokens[$classNamePointer];
        $className = $classNameToken['content'];

        $this->checkPrefix($phpcsFile, $classNamePointer, $className);
    }

    /**
     * Check for the prefix and add an error if it is absent.
     *
     * @param File $phpcsFile
     * @param int $stackPtr
     * @param string $className
     */
    protected function checkPrefix(File $phpcsFile, int $stackPtr, string $className)
    {
        $prefix = $this->getPrefix();

        $foundPrefix = \substr($className, 0, \strlen($prefix));

        if ($foundPrefix && \strtolower($foundPrefix) === \strtolower($prefix)) {
            return;
        }

        $phpcsFile->addError(
            'Missing prefix "%s" at "%s"',
            $stackPtr,
            $this->getErrorCode(),
            [
                $prefix,
                $className,
            ]
        );
    }

    /**
     * Return the prefix which should be checked.
     *
     * @return string
     */
    abstract protected function getPrefix(): string;

    /**
     * Return the error code which should be used by the sniff.
     *
     * @return string
     */
    abstract protected function getErrorCode(): string;
}
