<?php

declare(strict_types=1);

namespace DigiComp\PhpCodesniffer\DigiComp\Sniffs\ClassNames;

use PHP_CodeSniffer\Files\File;
use PHP_CodeSniffer\Sniffs\Sniff;
use SlevomatCodingStandard\Helpers\TokenHelper;

/**
 * Class AbstractMissingSuffixSniff is a blueprint for checking a missing suffix in a class name.
 */
abstract class AbstractMissingSuffixSniff implements Sniff
{
    /**
     * @inheritDoc
     */
    public function process(File $phpcsFile, $stackPtr)
    {
        $tokens = $phpcsFile->getTokens();

        $classNamePointer = TokenHelper::findNext(
            $phpcsFile,
            \T_STRING,
            $stackPtr + 1,
            $tokens[$stackPtr]['scope_opener']
        );
        $classNameToken = $tokens[$classNamePointer];
        $className = $classNameToken['content'];

        $this->checkSuffix($phpcsFile, $classNamePointer, $className);
    }

    /**
     * Check for the suffix and add an error if it is absent.
     *
     * @param File $phpcsFile
     * @param int $stackPtr
     * @param string $className
     */
    protected function checkSuffix(File $phpcsFile, int $stackPtr, string $className)
    {
        $suffix = $this->getSuffix();

        $foundSuffix = \substr($className, -\strlen($suffix));

        if (\strtolower($foundSuffix) === \strtolower($suffix)) {
            return;
        }

        $phpcsFile->addError(
            'Missing suffix "%s" at "%s"',
            $stackPtr,
            $this->getErrorCode(),
            [
                $suffix,
                $className,
            ]
        );
    }

    /**
     * Return the suffix which should be checked.
     *
     * @return string
     */
    abstract protected function getSuffix(): string;

    /**
     * Return the error code which should be used by the sniff.
     *
     * @return string
     */
    abstract protected function getErrorCode(): string;
}
