<?php

declare(strict_types=1);

namespace DigiComp\PhpCodesniffer\DigiComp\Sniffs\ClassNames;

use PHP_CodeSniffer\Files\File;

/**
 * Class MissingAbstractPrefixSniff checks if the class name of an abstract class begins with abstract.
 */
class MissingAbstractPrefixSniff extends AbstractMissingPrefixSniff
{
    public const CODE_MISSING_ABSTRACT_PREFIX = 'MissingAbstractPrefix';

    /**
     * @return int[]
     */
    public function register(): array
    {
        return [\T_CLASS];
    }

    /**
     * @inheritDoc
     */
    public function process(File $phpcsFile, $stackPtr)
    {
        $properties = $phpcsFile->getClassProperties($stackPtr);

        if (! $properties['is_abstract']) {
            return;
        }

        parent::process($phpcsFile, $stackPtr);
    }

    /**
     * @inheritDoc
     */
    protected function getPrefix(): string
    {
        return 'abstract';
    }

    /**
     * @inheritDoc
     */
    protected function getErrorCode(): string
    {
        return self::CODE_MISSING_ABSTRACT_PREFIX;
    }
}
